﻿using System;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.ComponentModel;

namespace DynamicTrackShapes
{
    public partial class  TestForm: Form
    {   

        public TestForm()
        {
            InitializeComponent();
        }

        private void TestForm_Load(object sender, EventArgs e)
        {
            BackgroundWorker worker = new BackgroundWorker();

            worker.DoWork += (s, args) => { LoadMap(); };

            worker.RunWorkerCompleted += (s, args) =>
            {
            };
            worker.RunWorkerAsync();
        }

     

        private void LoadMap()
        {
            
            winformsMap1.ExecuteOnOwningThread(() => winformsMap1.MapUnit = GeographyUnit.DecimalDegree);

            winformsMap1.ExecuteOnOwningThread(() => winformsMap1.CurrentExtent = new RectangleShape(-100.4, 50, -60, -10));

            //Adds the ECW
            EcwRasterLayer ecwImageLayer = new EcwRasterLayer(@"..\..\Data\World.ecw");
            ecwImageLayer.UpperThreshold = double.MaxValue;
            ecwImageLayer.LowerThreshold = 0;
            LayerOverlay imageOverlay = new LayerOverlay();
            imageOverlay.Layers.Add("EcwImageLayer", ecwImageLayer);
            winformsMap1.ExecuteOnOwningThread(() => winformsMap1.Overlays.Add(imageOverlay));

            winformsMap1.ExecuteOnOwningThread(() => winformsMap1.Refresh());
        }

   

        private void TrackEnded(object sender, TrackEndedTrackInteractiveOverlayEventArgs e)
        {   
            //Clears the Shapes from TrackOverlay.
            winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();
         
        }

        private void TrackOverlayCancelled()
        {
            ClearTrackingLayer();
            if (winformsMap1.TrackOverlay.TrackMode == TrackMode.Polygon)
            {
                winformsMap1.TrackOverlay.MouseClick(new InteractionArguments());
                winformsMap1.TrackOverlay.MouseDoubleClick(new InteractionArguments());
            }
            else if (winformsMap1.TrackOverlay.TrackMode == TrackMode.Line)
            {
                winformsMap1.TrackOverlay.MouseDoubleClick(new InteractionArguments());
            }
            else
            {
                winformsMap1.TrackOverlay.MouseUp(new InteractionArguments());
            }
            winformsMap1.TrackOverlay.TrackMode = TrackMode.None;


            winformsMap1.Refresh();

            
        }



        private void ClearTrackingLayer()
        {
            winformsMap1.TrackOverlay.Lock.EnterWriteLock();
            try
            {
                int count = winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Count;
                if (count > 0)
                {
                    winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();
                }
            }
            finally
            {
                winformsMap1.TrackOverlay.Lock.ExitWriteLock();
            }
        }


        private void TestForm_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Escape:
                    if (winformsMap1.TrackOverlay.TrackMode != TrackMode.None)
                    {
                        Cursor = Cursors.WaitCursor;
                        TrackOverlayCancelled();
                        winformsMap1.Refresh();
                        Cursor = Cursors.Default;
                    }
                    break;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            double lineLength = 0;
            if ( !double.TryParse(txtLength.Text.Trim(), out lineLength) )
            {
                MessageBox.Show("Enter a length");
                
                return;
            }

            TrackOverlayCancelled();

            winformsMap1.TrackOverlay = new TrackOverlayDemo() { currentLineLength = lineLength };  //  () { lineLength = 50 };
            winformsMap1.TrackOverlay.TrackMode = TrackMode.Line;
            
            
        }

    
     

   }
}
