﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Core;
using System.Collections.ObjectModel;

namespace DynamicTrackShapes
{
    //
    public class TrackOverlayDemo : TrackInteractiveOverlay
    {   
        public double currentLineLength;
        private PointShape point = new PointShape(0, 0);
        private InMemoryFeatureLayer internalFeatureLayer;

        public TrackOverlayDemo() : base()
        {
            internalFeatureLayer = TrackShapeLayer.CloneDeep() as InMemoryFeatureLayer;
        }

        protected override InteractiveResult MouseMoveCore(InteractionArguments interactionArguments)
        {
            point.X = interactionArguments.WorldX;
            point.Y = interactionArguments.WorldY;

            InteractiveResult result = base.MouseMoveCore(interactionArguments);

            if (TrackShapeLayer.InternalFeatures.Count > 0)
            {
                LineShape line = TrackShapeLayer.InternalFeatures[TrackShapeLayer.InternalFeatures.Count - 1].GetShape() as LineShape;

                //double currentLineLength = Math.Sqrt((point.X - line.Vertices[0].X) * (point.X - line.Vertices[0].X) + (point.Y - line.Vertices[0].Y) * (point.Y - line.Vertices[0].Y));

                currentLineLength = Math.Sqrt((point.X - line.Vertices[0].X) * (point.X - line.Vertices[0].X) + (point.Y - line.Vertices[0].Y) * (point.Y - line.Vertices[0].Y));

                double newY = (point.Y - line.Vertices[0].Y) * 50 / currentLineLength + line.Vertices[0].Y;
                double newX = (point.X - line.Vertices[0].X) * 50 / currentLineLength + line.Vertices[0].X;

                line.Vertices[1] = new Vertex(newX, newY);
                
                Feature temp = TrackShapeLayer.InternalFeatures[TrackShapeLayer.InternalFeatures.Count - 1];

                temp.WellKnownBinary = line.GetWellKnownBinary();

                
            }
            return result;
        }

        public override bool IsEmpty
        {
            get
            {
                if (internalFeatureLayer.InternalFeatures.Count == 0 && TrackShapeLayer.InternalFeatures.Count == 0)
                {
                    return true;
                }
                return false;
            }
        }

        protected override InteractiveResult MouseDoubleClickCore(InteractionArguments interactionArguments)
        {
            InteractiveResult result = base.MouseDoubleClickCore(interactionArguments);
            if (TrackShapeLayer.InternalFeatures.Count > 0)
            {
                LineShape line = TrackShapeLayer.InternalFeatures[TrackShapeLayer.InternalFeatures.Count - 1].GetShape() as LineShape;

                //double currentLineLength = Math.Sqrt((point.X - line.Vertices[0].X) * (point.X - line.Vertices[0].X) + (point.Y - line.Vertices[0].Y) * (point.Y - line.Vertices[0].Y));

                currentLineLength = Math.Sqrt((point.X - line.Vertices[0].X) * (point.X - line.Vertices[0].X) + (point.Y - line.Vertices[0].Y) * (point.Y - line.Vertices[0].Y));

                double newY = (point.Y - line.Vertices[0].Y) * 50 / currentLineLength + line.Vertices[0].Y;
                double newX = (point.X - line.Vertices[0].X) * 50 / currentLineLength + line.Vertices[0].X;

                line.Vertices[1] = new Vertex(newX, newY);

                Feature temp = TrackShapeLayer.InternalFeatures[TrackShapeLayer.InternalFeatures.Count - 1];

                temp.WellKnownBinary = line.GetWellKnownBinary();

            }
            internalFeatureLayer.InternalFeatures.Add(TrackShapeLayer.InternalFeatures[0]);
            TrackShapeLayer.InternalFeatures.Clear();

            return result;
        }

        protected override void DrawCore(GeoCanvas canvas)
        {
            internalFeatureLayer.Open();
            internalFeatureLayer.Draw(canvas, new Collection<SimpleCandidate>());
            base.DrawCore(canvas);
        }
    }
}



